<?php

/***************************************************************************
 *   Copyright (C) 2005 by Ferenc Veres   *
 *   lion@netngine.hu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
include('TranslationBackend.php');
include_once('export/ExporterFactory.php');

/**
* WebBackend is a gateway between the WebGUI.php (HTML parts) and the TranslationBackend (logic).
*
* This class handles the functions which move data from the web frontend to the
* translation backend functions on various events generated by the user. Data are
* passed by this class from web controls to the backend (from $_POST).
*/

class WebBackend extends TranslationBackend
{
	public $xStringStack;	/**< Comma separated list of stacked (bookmarked) strings. */
	public $xBrowserHelper; /**< Suffix for the browser string editor field: md5 of the string, so browsers will offer the same history for the same strigns. */
	public $chkSearchSelects; /**< Array containing selected items in the search results area (checkboxes) */
	
	/**
	* Create a new WebBackend for the GUI operations.
	*/
	
	function __construct()
	{
		parent::__construct();
	}

	/**
	* Handle events generated by the user on the GUI and move data from the GUI to the backend.
	*/
	
	public function EventHandler()
	{
		global $_POST;
		
		// Handle Events
		$this->Event = (empty($_POST["__Event"]) ? "" : $_POST["__Event"] );

		if($this->Event != "")
		{
			
			// Move web variables into local controls

			$this->xBrowserHelper = (empty($_POST["xBrowserHelper"]) ? "" : $_POST["xBrowserHelper"]);
			$this->txtEditString = (empty($_POST["txtEditString_".$this->xBrowserHelper]) ? "" : $_POST["txtEditString_".$this->xBrowserHelper] );
			$this->txtEditString  = trim($this->txtEditString);

			$this->txtFindText = (empty($_POST["txtFindText"]) ? "" : $_POST["txtFindText"] );
			$this->ddlFindPercent = (empty($_POST["ddlFindPercent"]) ? 60 : $_POST["ddlFindPercent"]);
			$this->xStringID = (empty($_POST["xStringID"]) ? 0 : $_POST["xStringID"]);
			$this->xProjectID = (empty($_POST["xProjectID"]) ? 0 : $_POST["xProjectID"]);
			$this->ddlLocaleID = (empty($_POST["xOldLocaleID"]) ? "" : $_POST["xOldLocaleID"]);
			$this->xOldLocaleID = (empty($_POST["xOldLocaleID"]) ? "" : $_POST["xOldLocaleID"]);
			$this->ddlSearchType = (empty($_POST["ddlSearchType"]) ? 0 : $_POST["ddlSearchType"]);
			$this->xStringStack = (empty($_POST["xStringStack"]) ? "" : $_POST["xStringStack"]);
			$this->xDelStringID = (empty($_POST["xDelStringID"]) ? "" : $_POST["xDelStringID"]);
			$this->xDelProjectID = (empty($_POST["xDelProjectID"]) ? "" : $_POST["xDelProjectID"]);

			// Store search results checkboxes if any checked (for "save to all checked" operation)
			if(empty($_POST["chkSearchSelects"]))
			{
				unset($this->chkSearchSelects);
			}
			else
			{
				$this->chkSearchSelects = $_POST["chkSearchSelects"];
			}

			// If no valid root yet, try with posted ddlLocaleID too.
			if($this->xProjectID == 0)
			{
				$this->InitLanguage();
			}

			//echo "EVENT:".$this->Event;
			
			switch($this->Event)
			{
				case "find":
					$this->Event_FindCustomString();
					break;
					
				case "editstring":
					$this->Event_EditString();
					break;
					
				case "deletestring":
					$this->Event_DeleteString();
					break;
					
				case "deleteproject":
					$this->Event_DeleteProject();
					break;
				
				case "savenextmissing":
				case "savepopstack":
				case "savenext":
				case "nextmissing":
				case "next":
				case "save":
				case "savetoallchecked":
					$this->Event_SaveNavigate();
					break;
					
				case "setproject":
					$this->Event_SetProject();
					break;

				case "setlocale":
					$this->Event_SetLocale();
					break;
					
				case "exportpack":
					$this->Event_ExportPack();
					break;

			}
			

			// If anything to edit, set browser input field suffix (for browser form cache)
			if(!empty($this->LocatedString))
			{
				$this->xBrowserHelper = md5($this->LocatedString->GetOriginal());
			}


		}
	}

	/**
	* Find a custom entered string
	*/
	
	private function Event_FindCustomString()
	{
		// Search the string
		$this->FindStrings();

		if($this->xStringID != 0)
		{
			$this->SyncToString();
		}
		else
		{
			$this->SyncToProject();
		}
	}


	/**
	* Take a string for editing
	*/
	
	private function Event_EditString()
	{
		// Load the selected string and related project paths
		$this->SyncToString();
		
		// Find similar strings
		if(!empty($this->LocatedString))
		{
			$this->ddlSearchType = 0;
			$this->txtFindText = $this->LocatedString->GetOriginal();
			$this->FindStrings();
		}
	}

	/**
	* Delete a dead string and navigate somewhere logical
	*/

	private function Event_DeleteString()
	{
		$this->DeleteString();
		
		if($this->xDelStringID == $this->xStringID)
		{
			// If it was the current string, go back to the first in this project.
			$this->xStringID = 0;
			$this->SyncToProject();
		}
		else
		{
			// Not current string, so sync back to that string
			$this->SyncToString();
		}

		// Search unchanged
		$this->FindStrings();
	}
	
	/**
	* Delete a dead project and navigate to parent
	*/

	private function Event_DeleteProject()
	{
		// Deletes project and returns its parent.
		$this->xProjectID = $this->DeleteProject($this->xDelProjectID);
		$this->xStringID = 0;
		$this->SyncToProject();

		// Search unchanged
		$this->FindStrings();
	}

	/**
	* Save the current string edit and/or navigate to next as user requested.
	*/
	
	private function Event_SaveNavigate()
	{
		// Save the string??
		if($this->Event == "savenext" ||
			$this->Event == "savenextmissing" ||
			$this->Event == "savepopstack" ||
			$this->Event == "save" ||
			$this->Event == "savetoallchecked")
		{
			$this->SaveString();
		}

		// Save to all checked search result values?
		if($this->Event == "savetoallchecked")
		{
			if(count($this->chkSearchSelects) > 0)
			{
				foreach($this->chkSearchSelects as $stringID)
				{
					// Load each string from the DB, change it and save changes
					$string = new TranslationString($stringID);
					$string->SetTranslation($this->txtEditString);
					$string->Save();
				}
			}
		}

		// How to find the next string?
		$locType = LOCATE_CURR;
		
		if($this->Event == "savenext" || $this->Event == "next")
		{
			$locType = LOCATE_NEXT;
		}
		else if($this->Event == "savenextmissing" || $this->Event == "nextmissing")
		{
			$locType = LOCATE_NEXTMISSING;
		}
		else if($this->Event == "savepopstack")
		{
			// Also locate curr, but we take the last string from stack!
			$locType = LOCATE_CURR;
			
			$stack = split(",", $this->xStringStack);
			$this->xStringID = array_pop($stack);
			$this->xStringStack = join($stack, ",");
		}
		
		// Locate it on the selected way
		$this->LocateString($locType);

		$this->SyncToString();
		
		// Find similar strings
		if(!empty($this->LocatedString))
		{
			$this->ddlSearchType = 0;
			$this->txtFindText = $this->LocatedString->GetOriginal();
			$this->FindStrings();
		}
	}

	/**
	* Select another project from the project tree
	*/
	
	private function Event_SetProject()
	{
		$this->SyncToProject();	
		
		// Find similar strings
		if(!empty($this->LocatedString))
		{
			$this->ddlSearchType = 0;
			$this->txtFindText = $this->LocatedString->GetOriginal();
			$this->FindStrings();
		}

	}

	/**
	* Set the selected locale and navigate back to its root project
	*/
	
	private function Event_SetLocale()
	{
		$this->xOldLocaleID = (empty($_POST["ddlLocaleID"]) ? "" : $_POST["ddlLocaleID"]);
		$this->ddlLocaleID = $this->xOldLocaleID;
		$this->InitLanguage();

		$this->SyncToProject();
	}

	/**
	* Export the current locale to config-export-path and pack into a zip file
	*/
	
	private function Event_ExportPack()
	{
		global $CONFIG;

		// Check if exporting is enabled, quit if not
		if(!$CONFIG['enable_web_export'])
		{
			throw new Exception("Web GUI export function is disabled in configuration.");
		}
		
		// Find selected locale's name
		$localeName = "";
		foreach(TranslationProject::$LocaleIDCache as $name => $id)
		{
			if($id == $this->ddlLocaleID)
			{
				$localeName = $name;
				break;
			}
		}

		if($localeName == "")
		{
			throw new Exception("Cannot find locale name for current locale id: " + $this->ddlLocaleID);
		}

		// Create a Xaraya exporter
		// TODO: (Hardcoded project type for now!)
		$factory = new ExporterFactory();
		$proc = $factory->NewExporter("Xaraya");

		// Export to the configured output location
		$proc->Export($localeName, true);
		
		// Compress the dir with TAR
		$languagePack = $localeName."_".date('Y-m-d').".tar.gz";
		system("tar -C ".$CONFIG['export_path']." --exclude $localeName/php -czf ".$CONFIG['language_pack_dir']."/$languagePack $localeName", $ret);
		if($ret != 0)
		{
			echo("<p>Tar returned $ret, the package may be incomplete.</p>");
		}
		$link = $CONFIG['language_pack_url']."/$languagePack";
		echo("<p>Language pack saved to: <a href=\"$link\">$link</a>.</p>");
	
		// Sync controls
		if($this->xStringID != 0)
		{
			$this->SyncToString();
		}
		else
		{
			$this->SyncToProject();
		}
	}

	/**
	* Function to sync all controls to the current project
	*/

	private function SyncToProject()
	{
		$this->xStringID = 0;
		
		if($this->xProjectID == 0)
		{
			return;
		}

		// Load the project's strings
		$this->LoadProjectStrings();
		
		if($this->CurrentStrings->valid())
		{
			// If there are strings, take the first one and Load it
			$this->xStringID = $this->CurrentStrings->current()->GetStringID();
			$this->LocateString();

			// Find similar strings
			$this->txtFindText = $this->LocatedString->GetOriginal();
			$this->FindStrings();
		}
		else
		{
			// Forget last string ID if no strings for this project.
			$this->xStringID = 0;
		}

		// Load related project tree too
		$this->FindProjectPath();
	}

	/**
	* Function to sync all controls to the current string
	*/
	
	private function SyncToString()
	{
		if($this->xStringID == 0)
		{
			return;
		}
		
		$this->LocateString();
		
		if(!empty($this->LocatedString))
		{
			// If found the string, load related project info
			$this->xProjectID = $this->LocatedString->GetProjectID();
			$this->LoadProjectStrings();
		}

		// Load related project tree too
		$this->FindProjectPath();
	}

}
