<?
/***************************************************************************
 *   Copyright (C) 2005 by Ferenc Veres   *
 *   lion@netngine.hu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
include_once('TranslationProject.php');

/**
* Class to list and iterate TranslationProject classes as they are stored in the database.
*
* ProjectIterator can be initialized with a parent project id, and then it can
* be used to loop on child projects, either recursve or direct childs.
*/

class ProjectIterator implements Iterator, RecursiveIterator
{
	private $LocaleID;		/**< Work with strings from this locale. */
	private $ParentID;		/**< Start listing childs of this project. */

	private $CurrentProject;	/**< ID of the current project in the Iterator. */

	public static $ProjectList;	/**< Cached list of all available projects */
	public static $ProjectChildren;	/**< Cached array to hold project tree data */

	/**
	* Create a new TranslationProject iterator for looping on child projects.
	*
	* @param $parentID The project ID in which we will loop child projects.
	* @param $localeID The locale to work with.
	*/
	public function __construct($parentID, $localeID)
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$this->LocaleID = $localeID;
		$this->ParentID = $parentID;
		
		// Cache all projects for the current language, unless done already
		if(empty(self::$ProjectList[$this->LocaleID]))
		{
			$rs = $DB->Execute("SELECT ID, ParentID, Name, LocaleID, Skeleton, OrigDoc,  TIMESTAMP(LastSeen) as LastSeen FROM ".$Tables["Projects"]." WHERE LocaleID = ? ORDER BY Name", $this->LocaleID);
			$haveMore = true;
			
			while($haveMore)
			{
				$proj = new TranslationProject();
				
				if(($haveMore = $proj->LoadFromRecordSet($rs)))
				{
					self::$ProjectList[$this->LocaleID][$proj->GetID()] = $proj;
					self::$ProjectChildren[$this->LocaleID][$proj->GetParentID()][] = $proj->GetID();
				}
			}
			$rs->Close();
		}
		
		if(empty(self::$ProjectList[$this->LocaleID]))
		{
			throw new Exception("There are no projects for Language ID: ".$this->LocaleID.". Please import before using.");
		}

		// Select the first child by default for ->valid()-ity check.
		$this->rewind();
	}

	/* Iterator interface */

	/**
	* Iterator interface: rewind function.
	*/
	public function rewind()
	{
		// Take first child
		if(!empty(self::$ProjectChildren[$this->LocaleID][$this->ParentID][0]))
		{
			$this->CurrentID = self::$ProjectChildren[$this->LocaleID][$this->ParentID][0];
		}
		else
		{
			$this->CurrentID = 0;
		}
	}
	/**
	* Iterator interface: hasMore function.
	*/
	public function hasMore()
	{
		// Get an array of our sibling' IDs
		$siblings = self::$ProjectChildren[$this->LocaleID][
			self::$ProjectList[$this->LocaleID][$this->CurrentID]->GetParentID()
			];

		// Loop on each sibling
		$takeNext = false;
		$foundMore = false;
		foreach($siblings as $pid)
		{
			// Found the next sibling?
			if($takeNext)
			{
				$foundMore = true;
				break;
			}

			// Is this the current item? If yes, take next sibling in the next round.
			if($pid == $this->CurrentID)
			{
				$takeNext = true;
			}
		}
		return $foundMore;
	}
	/**
	* Iterator interface: key function.
	*/
	public function key()
	{
		return $this->CurrentID;
	}
	/**
	* Iterator interface: current function.
	*/
	public function current()
	{
		return self::$ProjectList[$this->LocaleID][$this->CurrentID];
	}
	/**
	* Iterator interface: next function.
	*/
	public function next()
	{
		// Get an array of our sibling' IDs
		$siblings = self::$ProjectChildren[$this->LocaleID][
			self::$ProjectList[$this->LocaleID][$this->CurrentID]->GetParentID()
			];

		// Note the last ID and set like if we had no more (will change if we have)
		$lastID = $this->CurrentID;
		$this->CurrentID = 0;
		
		// Loop on each sibling
		$takeNext = false;
		foreach($siblings as $pid)
		{
			// Found the next sibling?
			if($takeNext)
			{
				$this->CurrentID = $pid;
				break;
			}
			
			// Is this the current item? If yes, take next sibling in the next round.
			if($pid == $lastID)
			{
				$takeNext = true;
			}
		}
	}
	/**
	* Iterator interface: valid function.
	*/
	public function valid()
	{
		return $this->CurrentID != 0;
	}

	/* Recursive Iterator Interface */

	/**
	* Recursive Iterator: hasChildren function
	*/
	public function hasChildren()
	{
		// Check if the children array has items with the current id as parent
		return !empty(self::$ProjectChildren[$this->LocaleID][$this->CurrentID]);
	}

	/**
	* Recursive Iterator: getChildren function
	*/
	public function getChildren()
	{
		// Make a new iterator below ourselves
		return new ProjectIterator($this->CurrentID, $this->LocaleID);
	}
}
?>